'------------------------------------------------------------------
'
'                           Imobilizer
'                          ------------
'
'           pinB.0                 vystup na rele,
'           pinB.1                 vystup na piezo,
'           pinD.0                 ID-12 (vyv. 9)
'           pinD.1          tlacitko, maze EEPROM (1k na +5V)
'           ma casove omezeni (10s)
'           pri nespravnem RFID lze 5x opakovat
'           obsahuje WatchDog
'
'------------------------------------------------------------------
'
'           Komercni vyuziti jen se souhlasem autora
'
'------------------------------------------------------------------
'
' Imobilizer po privedeni napajeni ma aktivni vystupni uroven rele,
' ceka 10 sekund na kod RFID, aby mohl stav vystupu podrzet. Pokud
' se kod nenacte (ani opakovane, max. 5 pokusu), ci po zapnuti
' ubehne cas delsi nez zminenych 10 sekund, vystupni uroven se zmeni
' na opacnou, cimz pripojene zarizeni zablokuje. Vse je doprovazeno
' zvukovou signalizaci.
' Programovani kodu se provadi u dosud nepripojeneho zarizeni postupnym
' prikladanim RFID. Po naprogramovani se provede reset a zarizeni je
' pripraveno k pouziti. Opetovne programovani se provede vymazanim
' stavajicich dat, to je pridrzenim tlacitka na DPS, zapnutim napajeni a
' po zazneni signalu jeho vypnutim. Dale se postupuje jako na zacatku.
'
'-------------------------------------------------------------------
$regfile = "2313def.dat"
$crystal = 8000000
$baud = 9600

Config Portb = Output
Config Portd = Input
Config Timer1 = Timer , Prescale = 256
Config Watchdog = 1024                                      'nastaveni watchdogu na 1024 milisekund
Start Watchdog

On Ovf1 Casovac
Enable Timer1
Enable Interrupts


Dim Cas As Byte                                             'mereni casu (po 0.1s) pro casove blokovani
Dim Pocet_odmitnuti As Byte                                 'pocet odmitnuti nespravnych RFID
Dim Vystupni_port As Byte                                   'prednastaveni portu
Dim Misto As Byte                                           'po nalezeni shodneho kodu (nacteny vuci prectenemu z EEPROM) ozacuje misto v pameti
Dim Dat(6) As Byte                                          'pole 6ti rozlozenych bytu pro komunikaci s EEPROM
Dim Offset_dat As Byte                                      'posun zacatku RFID kodu v EEPROM
Dim Adresa As Byte                                          'adresa v EEPROM
Dim Pocet_rfid As Byte                                      'pocet kodu RFID v EEPROM
Dim X As Byte                                               'promenna pro cyklus
Dim V As Byte                                               'promenna pro cyklus
Dim Z As Byte                                               'misto v retezci
Dim A As String * 14                                        'kod RFID nacteny ze snimace
Dim B As String * 12                                        'kod RFID nacteny z EEPROM
Dim Y As String * 2                                         '1 byte v retezci

Vystupni_port = 0                                           'nastaveni portu
Portb = Vystupni_port                                       'zapsani do portu
Gosub Start_povolen                                         'prednastaveni vystupu (start povolen), dal se ceka nastaveny cas na prilozeni RFID, pokud se prekroci cas, zarizeni se zablokuje

Tcnt1h = 243
Tcnt1l = 203

Start Timer1                                                'spusti casovac

Do
Gosub Vymazani_nactenych_kodu                               'maze kody v EEPROM
Gosub Cteni_poctu_rfid
If Pocet_rfid = 255 Then                                    'hlida, maji-li se programovat nove kody
   Stop Timer1                                              'zastavi casovac
   Stop Watchdog                                            'zastavi watchdog
   Goto Programovani_rfid                                   'programovani novych kodu do EEPROM
End If

Gosub Cteni_rfid                                            'nacte se snimacem kod RFID
Gosub Hledani                                               'hledani kodu v EEPROM
If Misto > 0 Then                                           'v EEPROM nalezen kod RFID
   Stop Timer1
   Do                                                       'zacykleni programu po uspesnem nalezeni kodu, vystup trvale odblokovan
   Reset Watchdog                                           'reset watchdogu, hlida cyklus aby neodpadlo rele
   Loop
   Else
   Gosub Signal                                             'signal zamitnuti
   Gosub Signal
   Gosub Signal
   Gosub Signal
End If
Incr Pocet_odmitnuti
Vystupni_port = 1                                           'pred zablokovanim vystupu pro rele drzi vystup aktivni
Portb = Vystupni_port
Gosub Restart_ctecky_id20                                   'provede restart obvodu ID-20, pro rychle nacteni noveho kodu
Loop Until Pocet_odmitnuti > 4                              'nastaveni poctu pokusu prihlaseni nespravnym RFID
Gosub Start_zakazan                                         'zablokovani vysupu po petinasobnem precteni nespravneho kodu
Stop Watchdog
End                                                         'Konec programu

'**********************   Podprogramy   **********************

Zapis_dat:                                                  'zapise data do EEPROM
For X = 1 To 6
Adresa = Offset_dat + X
Writeeeprom Dat(x) , Adresa
Next X
Return

Zapis_poctu_rfid:                                           'zapise pocet RFID na urcene misto v EEPROM
Writeeeprom Pocet_rfid , 0
Return

Cteni_eeprom:                                               'cte sekvenci kodu RFID z urceneho mista (offset)
For X = 1 To 6
Adresa = Offset_dat + X
Readeeprom Dat(x) , Adresa
Next X
Gosub Slozeni_retezce
Return

Cteni_poctu_rfid:                                           'precte pocet RFID z urceneho misto v EEPROM                                     '
Readeeprom Pocet_rfid , 0
Return

Rozlozeni_retezce:                                          'rozdeluje kod RFID na byty
For X = 1 To 6
Z = X * 2
Z = Z - 1
Y = Mid(a , Z , 2)
Dat(x) = Hexval(y)                                          'vystup prevodu
Next X
Return

Slozeni_retezce:                                            'slouci byty kodu do jednoho retezce
For X = 1 To 6
B = B + Hex(dat(x))
Next X
Return

Cteni_rfid:                                                 'nacte seriovy kod ze ctecky
A = ""
Input A
A = Right(a , 12)
Gosub Signal
Return

Signal:                                                     'vystup na piskle
Vystupni_port = Vystupni_port + 2
Portb = Vystupni_port
Waitms 50
Vystupni_port = Vystupni_port - 2
Portb = Vystupni_port
Return

Restart_ctecky_id20:                                        'provede reset ctecky
Vystupni_port = Vystupni_port + 4
Portb = Vystupni_port
Waitms 100
Vystupni_port = Vystupni_port - 4
Portb = Vystupni_port
Return

Hledani:                                                    'prohleda EEPROM, je-li tam shodny kod
Offset_dat = 0
Gosub Cteni_poctu_rfid
For V = 1 To Pocet_rfid
B = ""
Gosub Cteni_eeprom
Offset_dat = Offset_dat + 6
If B = A Then                                               'A - kod ze snimace; B - kod z EEPROM
   Misto = Offset_dat / 6                                   'do promenne "misto" se ulozi pozice nalezeneho kodu
End If
Next V
Return

Start_povolen:                                              'nastavi vystupni bit - zapne rele
Vystupni_port = Vystupni_port + 1
Portb = Vystupni_port
Return                                                      'nastaveni portu do odblokovaneho stavu

Start_zakazan:                                              'zrusi vystupni bit - vypne rele
Vystupni_port = Vystupni_port - 1
Portb = Vystupni_port
Return                                                      'nastaveni portu do blokujiciho stavu

Programovani_rfid:                                          'programovani novych RFID
Pocet_rfid = 1
Do
Gosub Cteni_rfid
Gosub Rozlozeni_retezce
Gosub Zapis_dat
Gosub Zapis_poctu_rfid
Incr Pocet_rfid
Offset_dat = Offset_dat + 6
Waitms 100                                                  'pauza pro lepsi rozeznani zvukoveho signalu
Gosub Signal
Loop Until Pocet_rfid > 21                                  'omezeni max poctu RFID v EEPROM (max. 21)
Cls
End                                                         'end program

Vymazani_nactenych_kodu:                                    'je-li stisknuto tlacitko, prepise se informace o poctu RFID na 255 (prazdna pamet)
If Pind.1 = 0 Then
   Pocet_rfid = 255
   Gosub Zapis_poctu_rfid
   Gosub Signal
   Waitms 100
   Gosub Signal
   Waitms 100
   Gosub Signal
   Stop Watchdog
   End                                                      'end program
End If
Return

Casovac:                                                    'hlida prekroceni casu
Reset Watchdog
Tcnt1h = 243
Tcnt1l = 203
Incr Cas
If Cas > 100 Then                                           'nastaveni casu (x0.1 s) po kterem se zarizeni zablokuje, pokud nenacte RFID
Gosub Start_zakazan
   Gosub Signal
   Gosub Signal
   Gosub Signal
   Gosub Signal
   Waitms 100
   Gosub Signal
   Gosub Signal
   Gosub Signal
   Gosub Signal
Do                                                          'zacykleni programu po preteceni casove ochrany, vystup trvale zablokovan
Reset Watchdog                                              'reset watchdogu, hlida cyklus aby bylo rele trvale zablokovane
Loop
End If
Return